import csv


def from_csv(file) -> list[list]:

    """
    Read CSV file into a list of lists.

    Takes a CSV file path, opens the file, and returns the 
    contents parsed into a list of rows, where each row 
    is a list of the data cells.

    :param file: Path to CSV file
    :type file: str
    :returns: Parsed CSV data
    :rtype: list[list]
    """

    with open(file, mode="r") as f:
        csv_file = csv.reader(f)
        data=[row for row in csv_file]
    return data



from toRST.toRST import Table
import pytest


@pytest.fixture
def empty_table():

    yield Table('tests/csv/empty.csv')

@pytest.fixture
def large_table():

    yield Table('tests/csv/large.csv')

@pytest.fixture
def small_table():

    yield Table('tests/csv/small.csv')

def test_headers(empty_table, large_table, small_table) -> None:

    assert empty_table.headers == ["No","Data","Here"]
    assert large_table.headers == ["First Name","Last Name","Job Title","Biography"]
    assert small_table.headers == ["Header1","Header2","Header3"]

def test_column_widths(empty_table, large_table, small_table) -> None:

    assert empty_table.column_widths[0] == len("No")
    assert large_table.column_widths[1] == len("Livingston")
    assert small_table.column_widths[2] == len("Header3")


from funcs.funcs import get_extension
from csv2rst.csv2rst import from_csv
from json2rst.json2rst import from_json


def handle_file(file: str) -> list[list]:

    extension = get_extension(file)
    if extension == "csv":
        return from_csv(file)
    elif extension == "json":
        return from_json(file)
    else:
        raise ValueError(f"{extension} files not currently supported")
class Table:

    def __init__(self, file) -> None:
        
        self.data = handle_file(file)
        self.headers = self.data[0]
        self.column_widths = self.get_column_widths()
        self.page_info = self.get_page_info()

    

    def get_column_widths(self) -> dict[int, int]:
        """
        Get the maximum width for each column.
        
        This determines the column widths needed to properly 
        format the reStructuredText grid by finding the max
        string length in each column of data.

        :returns: A dictionary with column index as key and width as value
        :rtype: dict[int, int]
    """
        column_widths = {}
        for index in range(int(len(self.headers))):
            max_width = max([len(str(row[index])) for row in self.data])
            column_widths[index] = max_width if max_width > len(self.headers[index]) else len(self.headers[index])
        return column_widths

    def get_page_info(self) -> dict[str, int or str]:
        """
        Get formatting information for the reST page.

        This returns a dictionary containing:

        - Number of columns
        - Total page width 
        - Separator string

        :returns: Dict with page formatting metadata
        :rtype: dict[str, int or str]
        """
        return {
            "num_columns": len(self.headers),
            "page_wdith": sum(self.column_widths.values()),
            "new_line": "".join(["+" + column_width * "=" for column_width in self.column_widths.values()]) + "+"
        }
    
    def build_table(self) -> list[str]:
        
        """
        Construct the reST grid table.

        This iterates through the data rows, building each line 
        of the table with proper column widths and formatting.

        :returns: A list of table lines
        :rtype: list[str]
        """
        table = []
        for row in self.data:
            
            table += [self.page_info["new_line"]]
            rst_row=""
            for index, cell in enumerate(row):
                column_width = self.column_widths[index]
                rst_row += "|" + (str(cell) + " " * (column_width - len(str(cell))))
            table += [rst_row + "|"]
        return table + [self.page_info["new_line"]]


from setuptools import setup, find_packages


readme = """
Command line tool for converting CSV and JSON files into reStructuredText Tables.
""".strip()

with open('LICENSE') as f:
    license = f.read()
setup(
    name='toRST',
    version='0.0.1',
    description='Command line tool for converting CSV and JSON files into reStructuredText Tables.',
    long_description=readme,
    author='John Reynolds',
    author_email='reynoldsjohngreg@gmail.com',
    url='https://github.com/jrey999/toRST',
    license=license,
    packages=find_packages(exclude=('tests',)),
    entry_points={
        'console_scripts': [
            'torst = toRST.write:torst'
        ]
    }
)


from toRST.toRST import Table
from funcs.command import parser, get_file_name


args = parser.parse_args()
def torst():
  
  for file in args.files:

    with open(f"{args.output_dir}/{get_file_name(file)}.rst", "w") as f:
        
        for line in Table(file).build_table():
            f.writelines(line + "\n")

if __name__ == "__main__":
    torst()